{---------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} UNIT Routines; {>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{---------------------------------------------------------------------------}


{***************************************************************************
 *                                                                         *
 *   Author     : Charlie Wiederhold                                       *
 *   Date       : January 7, 1996                                          *
 *   TA         : Gooty                                                    *
 *   TA Section : Wed. 3:00 - 4:00                                         *
 *   Assignment : Program 1 Inventory Tracker - UNIT Routines              *
 *                                                                         *
 ***************************************************************************}


{---------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} INTERFACE {>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{---------------------------------------------------------------------------}


USES
  Crt,          {* Used to do basic screen actions                *}
  Screen,       {* Prints to screen without interaction           *}
  Utility,      {* Performs basic screen routines/actions         *}
  List2Data,    {* Defines the types used                         *}
  List2ADT;     {* Abstract data type, use here as inventory list *}

{***************************************************************************}
FUNCTION  Convert
  (Strng : STRING;
   VAR Int : Integer) : Boolean;
  {* Purpose : Convert a string to integer, if possible        *}
  {*     Pre : String value to be changed                      *}
  {*    Post : Converted to integer or false funtion           *}

{***************************************************************************}
FUNCTION  ConvertReal
  (Strng : STRING;
   VAR Reel : Real): Boolean;
  {* Purpose : Convert a string to real, if possible           *}
  {*     Pre : String value to be changed                      *}
  {*    Post : Converted to real or false function             *}

{***************************************************************************}
FUNCTION  NewPart
  (Where : Byte) : Boolean;
  {* Purpose : Determine if part is new or invalid             *}
  {*    Post : True of false depending on user asnwer          *}

{***************************************************************************}
PROCEDURE ReadString
  (WhereX,
   WhereY : Integer;
   VAR Strng : STRING;
   Question : STRING);
  {* Purpose : Reads a string and records it in sales.log      *}
  {*    Post : String read and data stored in sales.log        *}

{***************************************************************************}
PROCEDURE Invalid
  (VAR Part : PartType);
  {* Purpose : Output invalid part                             *}
  {*    Post : Printout of invalid part                        *}

{***************************************************************************}
PROCEDURE InvalidPart
  (VAR Part : PartType);
  {* Purpose : Assign a part to the dummy invalid values       *}
  {*     Pre : Invalid part needing values for output          *}
  {*    Post : Part given dummy values                         *}

{***************************************************************************}
PROCEDURE InvalidData
  (Where : Byte;
   VAR Strng : STRING);
  {* Purpose : Prompt for response until valid answer is givin *}
  {*     Pre : Invalid answer                                  *}
  {*    Post : Valid answer                                    *}

{***************************************************************************}
PROCEDURE UpdateTotalPart
  (VAR Inventory : ListType;
   VAR Part : PartType);
  {* Purpose : Update an entire part from Update option        *}
  {*     Pre : Part has old data                               *}
  {*    Post : Part has current data                           *}

{***************************************************************************}
PROCEDURE UpdateOnHand
  (Amount : Integer;
   VAR Part : PartType;
   AddSub : Char);
  {* Purpose : Update the on hand total for a part             *}
  {*    Post : Part has new on hand total                      *}

{***************************************************************************}
PROCEDURE UpdatePart
  (Where : Byte;
   VAR Part : PartType);
  {* Purpose : Read part number,on hand, description           *}
  {*    Post : Part read for updating                          *}

{***************************************************************************}
PROCEDURE GetInfo
  (VAR Amount : Integer;
   VAR Number : PartString;
   AddOrSub : STRING);
  {* Purpose : Read info about a part                          *}
  {*    Post : Number and on hand values known for the part    *}

{***************************************************************************}
PROCEDURE ChangeInfo
  (VAR Inventory : ListType;
   VAR Part : PartType;
   Amount : Integer;
   AddSub : Char);
  {* Purpose : Change the information about a certain part     *}
  {*     Pre : Old info about part in storage                  *}
  {*    Post : New info about part in storage                  *}

{***************************************************************************}
PROCEDURE PrintOne
  (WhereHead,
   WherePart,
   WhereEnter : Byte;
   Part : PartType);
  {* Purpose : Print a header and one single part              *}
  {*    Post : Single part printed                             *}

{***************************************************************************}
PROCEDURE PrintReport
  (VAR Sales : SalesFile);
  {* Purpose : Print out final report in sales option          *}
  {*    Post : Final information printed with totals           *}

{***************************************************************************}
PROCEDURE WriteToSalesRpt
  (VAR Sales : SalesFile;
   Part : PartType;
   NumberSold : Integer);
  {* Purpose : Write a part to sales.rpt                       *}
  {*     Pre : Sale info in memory                             *}
  {*    Post : Sale info recorded in sales.rpt                 *}

{***************************************************************************}
PROCEDURE ReadSalesRpt
  (VAR Sales : SalesFile;
   VAR Part : PartType;
   VAR NumberSold : Integer);
  {* Purpose : Read a part from sales.rpt to calculate totals  *}
  {*     Pre : Part info unknown                               *}
  {*    Post : Part info known and useable                     *}

{***************************************************************************}


{---------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<} IMPLEMENTATION {>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{---------------------------------------------------------------------------}

{**************************************************************************}
FUNCTION Convert
  (Strng : STRING;
   VAR Int : Integer) : Boolean;
  {*     Pre : String value to be changed                      *}
  {*    Post : Converted to integer or false funtion           *}

VAR
  ErrorPos : Integer;  {* Error position *}

BEGIN {* Convert *}
  val(Strng, Int, ErrorPos);
  IF ErrorPos = 0 THEN
      Convert := True
    ELSE Convert := False;
  writeln (Log, 'Converting integer, if possible');
END; {* Convert *}

{**************************************************************************}
FUNCTION ConvertReal
  (Strng : STRING;
   VAR Reel : Real) : Boolean;
  {*     Pre : String value to be changed                      *}
  {*    Post : Converted to real or false function             *}

VAR
  DollarsStr,            {* Temp string for dollar amount *}
  CentsStr   : STRING;   {* Temp string for cent amoutn   *}
  Dollars,               {* Dollars in the price          *}
  Cents      : Integer;  {* Cents in the price            *}
  Long,                  {* Length of the string price    *}
  ErrorPos,              {* Error position                *}
  DecimalPos : Byte;     {* Decimal position              *}

BEGIN {* ConvertReal *}

  {* Break string into dollars and cents *}
  DollarsStr := Strng;
  CentsStr := Strng;
  Long := StrLength (Strng);
  DecimalPos := pos ('.',Strng);
  IF DecimalPos <> 0 THEN
      BEGIN
        Delete (DollarsStr,DecimalPos,Long);
        Delete (CentsStr,1,DecimalPos);
      END
    ELSE
      BEGIN
        DollarsStr := Strng;
        CentsStr := '00';
      END;

  {* Convert dollars and cents, if possible *}
  IF NOT (Convert (DollarsStr,Dollars) AND Convert (CentsStr,Cents)) THEN
      ConvertReal := False
    ELSE
      BEGIN
        Reel := (Dollars + Cents/100);
        ConvertReal := True;
      END;

  writeln (Log, 'Converting real number, if possible');

END; {* ConvertReal *}


{**************************************************************************}
FUNCTION NewPart
  (Where : Byte) : Boolean;
  {*    Post : True of false depending on user asnwer          *}

VAR
  Ans    : Char;       {* User char answer *}
  Answer : STRING;     {* Temp string for answer *}

BEGIN {* NewPart *}
  ReadString (25,Where,Answer,'Is this a new part? <Y/N> ==> ');
  Ans := Answer[1];
  Answer := upcase(Ans);
  IF Answer = 'Y' THEN
      NewPart := True
    ELSE NewPart := False;
END; {* NewPart *}

{**************************************************************************}
PROCEDURE ReadString
  (WhereX,
   WhereY : Integer;
   VAR Strng : STRING;
   Question : STRING);
  {*    Post : String read and data stored in sales.log        *}

BEGIN {* ReadString *}
  WrtS (WhereX,WhereY,Question);
  readln (Strng);
  writeln (Log, Question, Strng);
END; {* ReadString *}

{**************************************************************************}
PROCEDURE Invalid
  (VAR Part : PartType);
  {*    Post : Printout of invalid part                        *}

BEGIN {* Invalid *}
  writeln (Log, 'Invalid part entered');
  InvalidPart (Part);
  Beep (400,100);
END; {* Invalid *}

{**************************************************************************}
PROCEDURE InvalidPart
  (VAR Part : PartType);
  {*     Pre : Invalid part needing values for output          *}
  {*    Post : Part given dummy values                         *}

BEGIN {* InvalidPart *}
  writeln (Log, 'Dummy value for invalid part');
  Part.OnHand := 0;
  Part.Descr := 'Not Found: Try Again';
  Part.Wholesale := 0;
  Part.Retail := 0;
END; {* InvalidPart *}

{**************************************************************************}
PROCEDURE InvalidData
  (Where : Byte;
   VAR Strng : STRING);
  {*     Pre : Invalid answer                                  *}
  {*    Post : Valid answer                                    *}

BEGIN {* InvalidData *}
  writeln (Log, 'Invalid data entered');
  Clear (MainTextColor,MainBackColor,1,Where + 8,80,Where + 9);
  Beep (400,100);
  ReadString (3,1,Strng,'Invalid Number: Please enter again ==> ');
  window (1,9,80,24);
END; {* InvalidData *}

{**************************************************************************}
PROCEDURE UpdateTotalPart
  (VAR Inventory : ListType;
   VAR Part : PartType);
  {*     Pre : Part has old data                               *}
  {*    Post : Part has current data                           *}

VAR
  Strng : STRING;  {* Temp string value *}

BEGIN {* UpdateTotalPart *}

  {* Read on hand *}
  ReadString (3,3,Strng,'Change on hand to ==> ');
  IF Strng <> EmptyString THEN
      WHILE NOT Convert (Strng, Part.OnHand) DO
        InvalidData (3,Strng);

  {* Read wholsale price *}
  ReadString (3,4,Strng,'New wholesale price ==> $');
  IF Strng <> EmptyString THEN
      WHILE NOT ConvertReal (Strng,Part.Wholesale) DO
        InvalidData (4,Strng);

  {* Read retail price *}
  ReadString (3,5,Strng,'New retail price ==> $');
  IF Strng <> EmptyString THEN
      WHILE NOT ConvertReal (Strng,Part.Retail) DO
        InvalidData (5,Strng);

  {* Update data *}
  DeleteElement (Inventory, Part);
  InsertElement (Inventory, Part);

END; {* UpdateTotalPart *}

{**************************************************************************}
PROCEDURE UpdateOnHand
  (Amount : Integer;
   VAR Part : PartType;
   AddSub : Char);
  {*    Post : Part has new on hand total                      *}

BEGIN {* UpdateOnHand *}
  CASE AddSub OF
    'S' : Part.OnHand := Part.OnHand - Amount;
    'A' : Part.OnHand := Part.OnHand + Amount;
    'C' : Part.OnHand := Amount;
  END; {* CASE *}
  writeln (Log, 'On hand updated = ', Part.OnHand);
END; {* UpdateOnHand *}

{**************************************************************************}
PROCEDURE UpdatePart
  (Where : Byte;
   VAR Part : PartType);
  {*    Post : Part read for updating                          *}

VAR
  Strng : STRING;  {* Temp string value *}

BEGIN {* UpdatePart *}
  ReadString (25,Where,Strng,'Part description ==> ');
  Part.Descr := Strng;
  ReadString (25,Where + 1,Strng,'Wholesale price ==> $');
  WHILE NOT ConvertReal (Strng, Part.Wholesale) DO
    InvalidData (Where + 1,Strng);
  ReadString (25,Where + 2,Strng,'Retail price ==> $');
  WHILE NOT ConvertReal (Strng, Part.Retail) DO
    InvalidData (Where + 2,Strng);
END; {* UpdatePart *}

{**************************************************************************}
PROCEDURE GetInfo
  (VAR Amount : Integer;
   VAR Number : PartString;
   AddOrSub : STRING);
  {*    Post : Number and on hand values known for the part    *}

VAR
  Strng : STRING;  {* Temp string value *}

BEGIN {* GetInfo *}
  ReadString (3,2,Strng,AddOrSub);
  WHILE NOT Convert (Strng, Amount) DO
    InvalidData (2,Strng);
  ReadString (3,3,Strng,'Part number ==> ');
  Number := Strng;
END; {* GetInfo *}

{**************************************************************************}
PROCEDURE ChangeInfo
  (VAR Inventory : ListType;
   VAR Part : PartType;
   Amount : Integer;
   AddSub : Char);
  {*     Pre : Old info about part in storage                  *}
  {*    Post : New info about part in storage                  *}

BEGIN {* ChangeInfo *}
  DeleteElement (Inventory, Part);
  UpdateOnHand (Amount, Part, AddSub);
  writeln (Log, 'Part information changed');
  InsertElement (Inventory, Part);
END; {* ChangeInfo *}


{**************************************************************************}
PROCEDURE PrintOne
  (WhereHead,
   WherePart,
   WhereEnter : Byte;
   Part : PartType);
  {*    Post : Single part printed                             *}

BEGIN {* PrintOne *}
  Header (WhereHead);
  OutputPart (Part,WherePart);
  PressEnter (WhereEnter);
  writeln (Log, 'Single part outputed to screen');
END; {* PrintOne *}

{**************************************************************************}
PROCEDURE Totals
  (VAR Sales : SalesFile;
   Wholesale,
   Retail,
   Profits : Real);
  {* Purpose : Calculate and print totals to screen, sales.rpt *}
  {*     Pre : Totals unknown                                  *}
  {*    Post : Totals known and stored                         *}

BEGIN {* Totals *}

  {* Output totals *}
  Clear (MainTextColor,MainBackColor,1,9,80,24);
  Wrts (25,2,'Wholesale Costs ==> $');
  writeln (Wholesale:0:2);
  WrtS (25,3,'Retail totals   ==> $');
  writeln (Retail:0:2);
  WrtS (25,4,'---------------------------');
  WrtS (25,5,'Total Profits   ==> $');
  writeln (Profits:0:2);
  PressEnter (7);

  {* Write totals to sales.rpt *}
  append (Sales);
  writeln (Sales);
  writeln (Sales, 'Wholesale = $', Wholesale:0:2);
  writeln (Sales, 'Retail    = $', Retail:0:2);
  writeln (Sales, 'Profits   = $', Profits:0:2);
  writeln (Log, 'Totals for sales calculated');

END; {* Totals *}

{**************************************************************************}
PROCEDURE PrintReport
  (VAR Sales : SalesFile);
  {*    Post : Final information printed with totals           *}

VAR
  NumberSold,            {* Number of parts sold    *}
  Where      : Integer;  {* Where to write the info *}
  Part       : PartType; {* Part being used         *}
  Profits,               {* Profits made that day   *}
  Retail,                {* Retail sales            *}
  Wholesale  : Real;     {* Wholesale costs         *}

BEGIN {* PrintReport *}

  {* Initialize and get ready to read from sales.rpt *}
  assign (Sales, 'sales.rpt');
  reset (Sales);
  Clear (MainTextColor,MainBackColor,1,9,80,24);
  Header (1);
  Where := 3;
  Wholesale := 0;
  Retail := 0;
  Profits := 0;

  {* Read all parts from sales.rpt and output them *}
  WHILE NOT EOF(Sales) DO
    BEGIN {* WHILE *}
      IF (Where MOD 14 = 0) THEN
        BEGIN
          PressEnter (Where);
          Clear (MainTextColor,MainBackColor,1,9,80,24);
          Header (1);
          Where := 3;
        END;
      ReadSalesRpt (Sales, Part, NumberSold);

      {* Add up totals *}
      Wholesale := WholeSale + (Part.Wholesale * NumberSold);
      Retail := Retail + (Part.Retail * NumberSold);
      Profits := Profits + ((Part.Retail - Part.Wholesale) * NumberSold);
      OutputPart (Part,Where);
      inc(Where);
    END; {* WHILE *}

  {* Do final printouts and saving for FinalReport *}
  PressEnter (Where);
  Totals (Sales,Wholesale,Retail,Profits);
  Close (Sales);
  writeln (Log, 'Final sales report printed');

END; {* PrintReport *}

{**************************************************************************}
PROCEDURE WriteToSalesRpt
  (VAR Sales : SalesFile;
   Part : PartType;
   NumberSold : Integer);
  {*     Pre : Sale info in memory                             *}
  {*    Post : Sale info recorded in sales.rpt                 *}

BEGIN {* WriteToSalesRpt *}
  writeln (Sales, Part.OnHand);
  writeln (Sales, Part.Number);
  writeln (Sales, Part.Descr);
  writeln (Sales, Part.Wholesale:0:2);
  writeln (Sales, Part.Retail:0:2);
  writeln (Sales, NumberSold);
  writeln (Sales);
  writeln (Log, 'Wrote part to sales.rpt');
END; {* WriteToSalesRpt *}

{**************************************************************************}
PROCEDURE ReadSalesRpt
  (VAR Sales : SalesFile;
   VAR Part : PartType;
   VAR NumberSold : Integer);
  {*     Pre : Part info unknown                               *}
  {*    Post : Part info known and useable                     *}

BEGIN {* ReadSalesRpt *}
  readln (Sales, Part.OnHand);
  readln (Sales, Part.Number);
  readln (Sales, Part.Descr);
  readln (Sales, Part.Wholesale);
  readln (Sales, Part.Retail);
  readln (Sales, NumberSold);
  readln (Sales);
  writeln (Log, 'Read part from sales.rpt');
END; {* ReadSalesRpt *}

{**************************************************************************}

END. {* Routines *}
