PROGRAM PartInventoryTracker (Input, Output, PartFile, SalesFile, LogFile);

{***************************************************************************
 *                                                                         *
 *   Author     : Charlie Wiederhold                                       *
 *   Date       : January 7, 1996                                          *
 *   TA         : Gooty                                                    *
 *   TA Section : Wed. 3:00 - 4:00                                         *
 *   Assignment : Program 1 Inventory Tracker - Stub version               *
 *                                                                         *
 ***************************************************************************}

USES Crt,                       {* Used for ClrScr and other misc actions *}
     Utility,                   {* Performs many of the basic actions     *}
     Screen,                    {* Prints to screen without interaction   *}
     List2Data;                 {* Defines the data types to be used      *}

{**************************************************************************}
PROCEDURE Sales;
  {* Purpose : Read all sales made for the day, and insert data into *}
  {*           sales.rpt with final calculations                     *}
  {*     Pre : Sales.rpt is empty                                    *}
  {*    Post : Sales.rpt is full of all day's sales                  *}

BEGIN {* Sales *}
  writeln (Log, 'Procedure Sales called');
  WrtS (30,15,'Procedure Sales');
  readln;
END; {* Sales *}

{**************************************************************************}
PROCEDURE CheckStock;
  {* Purpose : To read a part number and print info if found         *}
  {*    Post : Part informartion printed if it exists                *}

BEGIN {* CheckStock *}
  writeln (Log, 'Procedure Check Stock called');
  WrtS (30,15,'Procedure CheckStock');
  readln;
END; {* CheckStock *}

{**************************************************************************}
PROCEDURE Reorder;
  {* Purpose : To take a threshhold and output a table of all parts  *}
  {*           that fall below that threshhold                       *}
  {*     Pre : Orders needed unknown                                 *}
  {*    Post : All parts needed ording printed to screen             *}

BEGIN {* Reorder *}
  writeln (Log, 'Procedure Reorder called');
  WrtS (30,15,'Procedure Reorder');
  readln;
END; {* Reorder *}

{**************************************************************************}
PROCEDURE NewStock;
  {* Purpose : To get new stock for existing and new parts           *}
  {*     Pre : Stock low, empty, or non-existent                     *}
  {*    Post : All new stock and parts added to inventory            *}

BEGIN {* NewStock *}
  writeln (Log, 'Procedure New Stock called');
  WrtS (30,15,'Procedure NewStock');
  readln;
END; {* NewStock *}

{**************************************************************************}
PROCEDURE Update;
  {* Purpose : Update existing parts                                 *}
  {*     Pre : Parts needing updating unchanged                      *}
  {*    Post : Parts needing updating updated to current values      *}

BEGIN {* Update *}
  writeln (Log, 'Procedure Update called');
  WrtS (30,15,'Procedure Update');
  readln;
END; {* Update *}

{**************************************************************************}
PROCEDURE OpenInventory;
  {* Purpose : To open all files and get inventory for program       *}
  {*     Pre : No information in inventory and all files closed      *}
  {*    Post : Inventory created and files opened and ready          *}

BEGIN {* OpenInventory *}
  assign (Log, 'sales.log');
  rewrite (Log);
  writeln (Log, 'Inventory will be opened, and program will start');
END; {* OpenInventory *}

{**************************************************************************}
PROCEDURE ProcessCommands;
  {* Purpose : Controls all selection of varios options              *}
  {*     Pre : Program just starting                                 *}
  {*    Post : Program finished and data updated                     *}

VAR
  Exit   : Boolean;  {* Boolean values controling exit condition   *}
  Cursor : byte;     {* Where the current selection is in the menu *}

BEGIN {* ProcessCommands *}

  {* Initializations and setup *}
  ScreenSetup;
  PrintMenu;
  Exit := FALSE;
  Cursor := 1;

  {* Runs every option till exit is selected *}
  WHILE NOT Exit DO
    BEGIN {* WHILE *}
      Select (MenuSelectColor,MenuBackColor,7,6,34,2,Cursor);
      TcTb (MainTextColor,MainBackColor);
      CASE Cursor OF
        1 : Sales;
        2 : CheckStock;
        3 : Reorder;
        4 : NewStock;
        5 : Update;
        6 : Exit := TRUE;
      END; {* CASE *}
      Clear (MainTextColor,MainBackColor,1,9,80,25);
      window (1,1,80,25);
    END; {* WHILE *}

END; {* ProcessCommands *}

{**************************************************************************}
PROCEDURE CloseInventory;
  {* Purpose : Save parts.bin and sales.log and other last actions   *}
  {*     Pre : Parts.bin and sales.log unfinished                    *}
  {*    Post : Parts.bin and sales.log saved and closed              *}

BEGIN {* CloseInventory *}
  write (Log, 'Inventory will be closed, and program ended');
  close (Log);
END; {* CloseInventory *}

{**************************************************************************}

BEGIN {* Main *}
  OpenInventory;
  ProcessCommands;
  CloseInventory;
END. {* Main *}
