UNIT CircleADT;

INTERFACE

USES
  Prog3Data;

TYPE
  PointerType = ^NodeType;
  NodeType = RECORD
    Info   : ListElementType;               (* list user's data *)
    Next   : PointerType                (* pointer to next node *)
  END; (* NodeType *)
  ListType = PointerType;


(*************************************************************)
PROCEDURE CreateList
  (VAR List : ListType);
  (* Function: Initializes List to empty state. *)
  (* Pre:  List has not been created.           *)
  (* Post: List exists and is empty.            *)

(*************************************************************)
PROCEDURE DestroyList
  (VAR List : ListType);
  (* Function: Destroys all elements, leaving List *)
  (*   in empty state.                             *)
  (* Pre:  List has been created.                  *)
  (* Post: List is empty.                          *)

(*************************************************************)
PROCEDURE InsertElement
  (VAR List : ListType;
   NewElement : ListElementType);
  {* Purpose : To insert a new element into the list *}
  {*     Pre : List has been created                 *}
  {*    Post : New element in list                   *}

(*************************************************************)
PROCEDURE GetNextItem
  (VAR CurrentElement : ListType;
   VAR NextElement : ListElementType);
  (* Function: Get the successor of current element.      *)

  (* Pre:  List has been created.                         *)
  (*       Current element has been defined.              *)
  (* Post: Current element = successor of current element *)
  (*       Element = current element                      *)

(*************************************************************)
(*-----------------------------------------------------------*)
(*************************************************************)

IMPLEMENTATION

(*************************************************************)
PROCEDURE CreateList
  (VAR List : ListType);
  (* Pre:  List has not been created.  *)
  (* Post: List.ListData is set to NIL. *)
BEGIN (* CreateList *)
  List := NIL;
END; (* CreateList *)

(*************************************************************)
PROCEDURE DestroyList
  (VAR List : ListType);
  (* Pre:  List has been created *)
  (* Post: List.Length = 0; List.ListData = NIL            *)
  (*       All items in List.ListData have been disposed.  *)
VAR
  CurrentPtr,
  TempPtr    : PointerType;

BEGIN (* DestroyList *)

  CurrentPtr := List^.Next;

  REPEAT
    TempPtr := CurrentPtr;
    CurrentPtr := CurrentPtr^.Next;
    Dispose (TempPtr);
  UNTIL CurrentPtr = List;

END; (* DestroyList *)

(*************************************************************)
PROCEDURE InsertElement
  (VAR List : ListType;
   NewElement : ListElementType);
  {* Purpose : To insert a new element into the list *}
  {*     Pre : List has been created                 *}
  {*    Post : New element in list                   *}
  (* NOTE:  List points to the last node in the list *)

VAR
  NewNode : PointerType; (* pointer to the new node *)
  PredLoc : PointerType; (* pointer to predecessor  *)

BEGIN (* InsertElement *)

  {* Creat e the new node *}
  New(NewNode);
  NewNode^.Info := NewElement;
  PredLoc := List;

  {* Insert it into the end of the list and update all pointers *}
  IF List <> NIL
     THEN  (* Find insertion point and link up. *)
       BEGIN
         NewNode^.Next := PredLoc^.Next;
         PredLoc^.Next := NewNode;
         List := NewNode
       END
     ELSE  (* We are inserting into an empty list. *)
       BEGIN
         NewNode^.Next := NewNode;
         List := NewNode;
       END;

END; (* InsertElement *)

(*************************************************************)
PROCEDURE GetNextItem
  (VAR CurrentElement : ListType;
   VAR NextElement : ListElementType);
  (* Pre:  List has been created.                    *)
  (*       List.CurrentItem has been initialized.    *)
  (*       GetNextItem is not called when            *)
  (*         List.CurrentItem is NIL.                *)
  (* Post: Element is set to List.CurrentItem^.Info. *)
  (*       CurrentItem is set to CurrentItem^.Next.  *)

BEGIN  (* GetNextItem *)
  NextElement := CurrentElement^.Info;
  CurrentElement := CurrentElement^.Next;
END;   (* GetNextItem *)

(*************************************************************)

END. {* IOServer *}