                 {* Charlie Wiederhold - Gooty Wed. 3-4 *}
{---------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} UNIT Advanced; {>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{---------------------------------------------------------------------------}


{---------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<} INTERFACE {>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{---------------------------------------------------------------------------}


USES
  Crt,          {* Many screen and interface functions                     *}
  Utility,      {* Contains my most used routines                          *}
  Prog4Data,    {* Declarations of the main data types                     *}
  TreeData,     {* Declaration of the Tree data element                    *}
  ListData,     {* Declaration of the List data element                    *}
  TreeADT,      {* The ADT for the search trees                            *}
  ListADT,      {* The ADT for the lists                                   *}
  Basics;       {* The basic routines for Program4                         *}

{***************************************************************************}
PROCEDURE UpdateTree
  (    KeyValue : STRING;            {* Key value to search by             *}
       NewNode,                      {* NewNode to be inserted             *}
       OldNode  : TreeElementType;   {* Currently existing node            *}
       BookList : ListType;          {* Master list of books               *}
   VAR Tree     : TreeType);         {* Tree to be inserted into           *}
{* Purpse : To update current values with a tree                           *}
{* Pre    : New values are not in the tree                                 *}
{* Post   : New values in the tree and in correct position                 *}

{***************************************************************************}
PROCEDURE CreateTitleTree
  (    Tree    : TreeType;           {* Tree to be traversed to get values *}
   VAR NewTree : TreeType);          {* New tree to be created             *}
{* Purpse : To create the title tree from the author tree                  *}
{* Pre    : Title tree is undefined                                        *}
{* Post   : Title tree is created and relatively balanced                  *}

{***************************************************************************}
PROCEDURE GetBooks
  (VAR Tree      : TreeType;         {* Tree to be created first           *}
   VAR BookList  : ListType;         {* Master list of books               *}
   VAR BookFile  : Text;             {* File the books are saved on        *}
   VAR CurrentID : Integer);         {* Current ID of the last book        *}
{* Purpse : To get the books from the file, create AuthorTree and BookList *}
{* Pre    : All data types are empty                                       *}
{* Post   : Author tree and Master BookList are fully created              *}

{***************************************************************************}
PROCEDURE SaveBooks
  (VAR Tree     : TreeType;          {* Tree to be saved from              *}
   VAR BookFile : Text);             {* File the books are saved on        *}
{* Purpse : To save the books to the file via the title tree               *}
{* Pre    : Books still loaded only in memory                              *}
{* Post   : Books saved onto the hard drive for later access               *}

{***************************************************************************}
PROCEDURE CheckBoth
  (    SearchBook : ListElementType; {* Search book entered by user        *}
   VAR BookList   : ListType;        {* Master list of books               *}
   VAR Trees      : IndexTrees;      {* Trees to be checked for values     *}
       Process    : ProcessMethod;   {* What process to perform            *}
   VAR LogFile    : Text);           {* LogFile                            *}
{* Purpse : To search for a book via title and author                      *}
{* Pre    : Position of search book unknown                                *}
{* Post   : Position (if it exists) is known                               *}

{***************************************************************************}
PROCEDURE CheckTitle
  (    SearchBook : ListElementType; {* Search book entered by user        *}
   VAR BookList   : ListType;        {* Master list of books               *}
   VAR Trees      : IndexTrees;      {* Trees to be checked for values     *}
       Process    : ProcessMethod;   {* What process to perform            *}
   VAR LogFile    : Text);           {* LogFile                            *}
{* Purpse : To search for a book via title                                 *}
{* Pre    : Position of search book unknown                                *}
{* Post   : Position (if it exists) is known                               *}

{***************************************************************************}
PROCEDURE CheckAuthor
  (    SearchBook : ListElementType; {* Search book entered by user        *}
   VAR BookList   : ListType;        {* Master list of books               *}
   VAR Trees      : IndexTrees;      {* Trees to be checked for values     *}
       Process    : ProcessMethod;   {* What process to perform            *}
   VAR LogFile    : Text);           {* LogFile                            *}
{* Purpse : To search for a book via author                                *}
{* Pre    : Position of search book unknown                                *}
{* Post   : Position (if it exists) is known                               *}

{***************************************************************************}


{---------------------------------------------------------------------------}
{<<<<<<<<<<<<<<<<<<<<<<<<<<<<} IMPLEMENTATION{>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>}
{---------------------------------------------------------------------------}


{***************************************************************************}
PROCEDURE UpdateTree
  (    KeyValue : STRING;            {* Key value to search by             *}
       NewNode,                      {* NewNode to be inserted             *}
       OldNode  : TreeElementType;   {* Currently existing node            *}
       BookList : ListType;          {* Master list of books               *}
   VAR Tree     : TreeType);         {* Tree to be inserted into           *}
{* Purpse : To update current values with a tree                           *}
{* Pre    : New values are not in the tree                                 *}
{* Post   : New values in the tree and in correct position                 *}

VAR
  Found : Boolean;                {* Boolean checking value                *}

BEGIN {* UpdateTree *}

  {* Initialize values *}
  NewNode.Key := KeyValue;
  OldNode.Key := KeyValue;
  RetrieveTreeElement (Tree,OldNode,Found);

  {* Check to see if it is new or needs to just be updated *}
  IF NOT(Found) THEN
      BEGIN {* If *}
        CreateList (NewNode.BookList);
        InsertElement (NewNode.BookList,BookList.CurrentItem^.Info);
        InsertTreeElement (Tree,NewNode);
      END {* If *}
    ELSE
      BEGIN {* Else *}
        InsertElement (OldNode.BookList,BookList.CurrentItem^.Info);
        ReplaceTreeElement(Tree,OldNode);
      END; {* Else *}

END; {* UpdateTree *}

{***************************************************************************}
PROCEDURE CreateTitleTree
  (    Tree    : TreeType;           {* Tree to be traversed to get values *}
   VAR NewTree : TreeType);          {* New tree to be created             *}
{* Purpse : To create the title tree from the author tree                  *}
{* Pre    : Title tree is undefined                                        *}
{* Post   : Title tree is created and relatively balanced                  *}

VAR
  NewNode  : TreeElementType;        {* New node to be checked and created *}
  Found    : Boolean;                {* Boolean checking value             *}

BEGIN {* CreateTitleTree *}

  {* Continues along the left sides of the tree *}
  IF Tree^.Left <> NIL THEN CreateTitleTree (Tree^.Left,NewTree);

  {* Reset the list and go through the entire thing *}
  ResetList (Tree^.Info.BookList);

  WHILE Tree^.Info.BookList.CurrentItem <> NIL DO
    BEGIN {* While *}
      NewNode.Key := Tree^.Info.BookList.CurrentItem^.Info.BookNode.Title;
      RetrieveTreeElement (NewTree,NewNode,Found);

      {* If it is new it creates the new node, or it just updates the old *}
      IF NOT(Found) THEN
          BEGIN {* If *}
            CreateList (NewNode.BookList);
            InsertElement (NewNode.BookList,Tree^.Info.BookList.CurrentItem^.Info);
            InsertTreeElement (NewTree,NewNode);
          END {* If *}
        ELSE
          BEGIN {* Else *}
            RetrieveElement (NewNode.BookList, Tree^.Info.BookList.CurrentItem^.Info, Found);
            IF NOT(Found) THEN
                BEGIN {* If *}
                  InsertElement (NewNode.BookList,Tree^.Info.BookList.CurrentItem^.Info);
                  ReplaceTreeElement (NewTree,NewNode);
                END; {* IF *}
          END; {* Else *}

      {* Move on to the next item *}
      Tree^.Info.BookList.CurrentItem := Tree^.Info.BookList.CurrentItem^.Next;
    END; {* While *}

  {* Continues along the right side of the tree *}
  IF Tree^.Right <> NIL THEN CreateTitleTree (Tree^.Right,NewTree)

END; {* CreateTitleTree *}

{***************************************************************************}
PROCEDURE GetBooks
  (VAR Tree      : TreeType;         {* Tree to be created first           *}
   VAR BookList  : ListType;         {* Master list of books               *}
   VAR BookFile  : Text;             {* File the books are saved on        *}
   VAR CurrentID : Integer);         {* Current ID of the last book        *}
{* Purpse : To get the books from the file, create AuthorTree and BookList *}
{* Pre    : All data types are empty                                       *}
{* Post   : Author tree and Master BookList are fully created              *}

VAR
  OldNode,                           {* Old node if it exist in the tree   *}
  NewNode     : TreeElementType;     {* New node if it doesn't exist       *}
  Book        : ListElementType;     {* New book to be entered             *}
  Count       : Integer;             {* Look counter variable              *}
  AuthorNames : AuthorArray;         {* Arrar of author names              *}

BEGIN {* GetBooks *}

  {* Initializations *}
  assign (BookFile, 'books.cat');
  reset (BookFile);
  Book.Element := ListPart;
  readln (BookFile);
  WHILE NOT Eof(BookFile) DO
    BEGIN {* While *}

      {* Read in each book and update the program information on it *}
      GetBookFile (Book,BookFile,CurrentID);
      InsertElement (BookList, Book);
      ResetList (BookList);
      BreakUpAuthors (Book.BookNode.Authors,AuthorNames);
      Count := 1;
      WHILE AuthorNames[Count] <> '' DO
        BEGIN {* While *}
          UpdateTree (AuthorNames[Count],NewNode,OldNode,BookList,Tree);
          inc(Count);
        END; {* While *}
    END; {* While *}

  {* Finish the file *}
  close (BookFile);

END; {* GetBooks *}

{***************************************************************************}
PROCEDURE SaveBookNodes
  (VAR Tree     : TreeType;          {* Tree traversed to save from        *}
   VAR BookFile : Text);             {* File to save the books to          *}
{* Purpse : To recursively travers the tree and save every node            *}
{* Pre    : Nodes unsaved in BookFile                                      *}
{* Post   : Nodes saved into BookFile                                      *}


VAR
  Node  : ListElementType;           {* Node to be checked in lists        *}

BEGIN {* SaveBookNodes *}

  {* Continue along the left side of the tree *}
  IF Tree^.Left <> NIL THEN SaveBookNodes (Tree^.Left,BookFile);

  {* Saves every node in the list, thus saving every book *}
  ResetList (Tree^.Info.BookList);
  WHILE Tree^.Info.BookList.CurrentItem <> NIL DO
    BEGIN {* For *}
      Node := Tree^.Info.BookList.Currentitem^.Info;
      writeln (BookFile,Break);
      writeln (BookFile,TitleHead,Node.BookNode.Title);
      writeln (BookFile,AuthorHead,Node.BookNode.Authors);
      Tree^.Info.BookList.CurrentItem := Tree^.Info.BookList.CurrentItem^.Next;
    END; {* For *}

  {* Continue along the right side of the tree *}
  IF Tree^.Right <> NIL THEN SaveBookNodes (Tree^.Right,BookFile)

END; {* SaveBookNodes *}

{***************************************************************************}
PROCEDURE SaveBooks
  (VAR Tree     : TreeType;          {* Tree to be saved from              *}
   VAR BookFile : Text);             {* File the books are saved on        *}
{* Purpse : To save the books to the file via the title tree               *}
{* Pre    : Books still loaded only in memory                              *}
{* Post   : Books saved onto the hard drive for later access               *}

BEGIN {* SaveBooks *}

  {* Simly sets up the file and calls the saving procedure *}
  assign (BookFile, 'books.cat');
  rewrite (BookFile);
  SaveBookNodes (Tree,BookFile);
  writeln (BookFile,Break);
  close (BookFile);

END; {* SaveBooks *}

{***************************************************************************}
PROCEDURE DeleteBookNode
  (    DeletedBook : ListElementType;{* Book to be deleted                 *}
       KeyValue    : STRING;         {* Key value to look for              *}
   VAR Tree        : TreeType;       {* Tree to be deleted from            *}
   VAR LogFile     : Text);          {* LogFile                            *}
{* Purpse : To delete a book from a tree                                   *}
{* Pre    : Book is a node of the tree                                     *}
{* Post   : Book has been deleted from the tree                            *}

VAR
  Found       : Boolean;             {* Boolean checking value                *}
  DeletedNode : TreeElementType;     {* Node to be removed from program    *}

BEGIN {* DeleteBookNode *}

  {* Delete from the lists, and the trees if the list is empty *}
  DeletedNode.Key := KeyValue;
  RetrieveTreeElement (Tree,DeletedNode,Found);
  DeleteElement (DeletedNode.BookList,DeletedBook);
  ReplaceTreeElement (Tree,DeletedNode);
  IF EmptyList(DeletedNode.BookList) THEN
       DeleteTreeElement (Tree,DeletedNode);

END; {* DeleteBookNode *}

{***************************************************************************}
PROCEDURE PerformDeletion
  (    DeletedBook : ListElementType;{* Goes though and finds delete point *}
       AuthorNames : AuthorArray;    {* Names of all the authors           *}
   VAR Trees       : IndexTrees;     {* Trees to be deleted from           *}
   VAR BookList    : ListType;       {* Master list of books               *}
   VAR LogFile     : Text);          {* LogFile                            *}
{* Purpse : To find the position to delete from                            *}
{* Pre    : Deletion spot is unkown                                        *}
{* Post   : Spot found and book is deleted from entire program             *}


VAR
  Count : Integer;                   {* Loop control variable              *}

BEGIN {* PerformDeletion *}


  {* Simply sets it up to pinpoint the node and delete it *}
  DeleteBookNode (DeletedBook,DeletedBook.BookNode.Title,Trees[Title],LogFile);
  BreakUpAuthors (DeletedBook.BookNode.Authors,AuthorNames);
  Count := 1;
  WHILE AuthorNames[Count] <> '' DO
     BEGIN {* While *}
       DeleteBookNode (DeletedBook,AuthorNames[Count],Trees[Author],LogFile);
       inc(Count);
     END; {* While *}
  DeleteElement (BookList,DeletedBook);

  writeln (LogFile,'This book was deleted');

END; {* PerformDeletion *}

{***************************************************************************}
PROCEDURE DeleteBook
  (    SearchNode  : TreeElementType;{* Node to delete from the program    *}
       Info        : InfoType;       {* What type of info the user game    *}
       NeededNames : STRING;         {* Names of authors needed for match  *}
   VAR BookList    : ListType;       {* Master list of books               *}
   VAR Trees       : IndexTrees;     {* Trees to be deleted from           *}
   VAR LogFile     : Text);          {* LogFile                            *}
{* Purpse : To read in a book and delete it from the program               *}
{* Pre    : Deletion book unknown                                          *}
{* Post   : Book known and deleted if it exits                             *}

VAR
  DeletedBook : ListElementType;     {* Book to be deleted                 *}
  DoDelete    : Boolean;             {* Boolean checking value             *}
  AuthorNames : AuthorArray;         {* Array of author names              *}

BEGIN {* DeleteBook *}

  {* Traverses the entire list to locate the book *}
  ResetList (SearchNode.BookList);
  WHILE SearchNode.BookList.CurrentItem <> NIL DO
    BEGIN {* While *}
      clrscr;

      {* Finds a match *}
      IF ((SearchNode.BookList.CurrentItem^.Info.BookNode.Authors = NeededNames) AND
         (Info = Both)) OR (Info = One) THEN
          BEGIN {* If *}

            {* Outputs and checks to see if it should be deleted *}
            DeletedBook := SearchNode.BookList.CurrentItem^.Info;
            OutputBook (DeletedBook,LogFile);
            writeln (' Do you want to delete this book? <Y/N>');
            HideCursor (1,1,WinColor);
            DoDelete := AskUserInfo;
            TcTb (TextColor,WinColor);
            IF DoDelete THEN
                PerformDeletion (DeletedBook,AuthorNames,Trees,BookList,LogFile);
          END; {* If *}

      {* Move on to the next node *}
      SearchNode.BookList.CurrentItem := SearchNode.BookList.CurrentItem^.Next;
    END; {* While *}

END; {* DeleteBook *}

{***************************************************************************}
PROCEDURE OutputBookElement
  (SearchNode  : TreeElementType;    {* Node to search for                 *}
   Info        : InfoType;           {* What type of info the user gave    *}
   NeededNames : STRING;            {* Names needed for a match           *}
   VAR LogFile : Text);              {* LogFile                            *}
{* Purpse : To output all the book elements matching the request           *}
{* Pre    : Books not known                                                *}
{* Post   : Books outputed to the screen if they are know                  *}

BEGIN {* OutputBookElement *}

  {* Traverses the entire list to find the book *}
  ResetList (SearchNode.BookList);
  WHILE SearchNode.BookList.CurrentItem <> NIL DO
    BEGIN {* While *}
      clrscr;

      {* Finds all matches... *}
      IF ((SearchNode.BookList.CurrentItem^.Info.BookNode.Authors = NeededNames) AND
         (Info = Both)) OR (Info = One) THEN
          BEGIN {* If *}

            {* .... and outputs them to the screen *}
            OutputBook (SearchNode.BookList.CurrentItem^.Info,LogFIle);
            IF (SearchNode.BookList.CurrentItem^.Next <> NIL) AND (Info = ONE) THEN
                writeln (' Press Enter to continue...')
              ELSE
                IF (SearchNode.BookList.CurrentItem^.Next = NIL) OR (Info = Both) THEN
                    writeln (' All books have been displayed. Press Enter to finish...');
            HideCursor (1,1,WinColor);
            readln;
            TcTb (TextColor,WinColor);
          END; {* If *}

      {* Move on the the next one *}
      SearchNode.BookList.CurrentItem := SearchNode.BookList.CurrentItem^.Next;
    END; {* While *}

END; {* OutputBookElement *}

{***************************************************************************}
FUNCTION Match
  (    SearchNode  : TreeElementType;{* Node in the tree searching         *}
       SearchBook  : ListElementType;{* Book to be searching for           *}
   VAR CheckBook   : ListElementType;{* Vrification book                   *}
       AuthorNames : AuthorArray) : Boolean;{* Array of all author names   *}
{* Purpse : To find if the names are matches for the search item           *}
{* Pre    : Match unkown                                                   *}
{* Post   : Matches known and sent back to main call                       *}

VAR
  Found : Boolean;                   {* Boolean checking value             *}
  Count : Integer;                   {* Loop control variable              *}

BEGIN {* Match *}

  {* Checks to see if there is a match for the given book value *}
  GetNextItem (SearchNode.BookList,CheckBook);
  Found := True;
  Count := 1;
  BreakUpAuthors (SearchBook.BookNode.Authors,AuthorNames);

  {* Checks all names *}
  WHILE (AuthorNames[Count] <> '') AND Found DO
    BEGIN {* While *}
      IF pos(AuthorNames[Count],CheckBook.BookNode.Authors) = 0 THEN
          Found := False;
      inc(Count);
    END; {* While *}
  Match := Found;

END; {* Match *}

{***************************************************************************}
PROCEDURE CheckBoth
  (    SearchBook : ListElementType; {* Search book entered by user        *}
   VAR BookList   : ListType;        {* Master list of books               *}
   VAR Trees      : IndexTrees;      {* Trees to be checked for values     *}
       Process    : ProcessMethod;   {* What process to perform            *}
   VAR LogFile    : Text);           {* LogFile                            *}
{* Purpse : To search for a book via title and author                      *}
{* Pre    : Position of search book unknown                                *}
{* Post   : Position (if it exists) is known                               *}

VAR
  CheckBook   : ListElementType;     {* Node to be checked and compared    *}
  SearchNode  : TreeElementType;     {* Node to be searched for if exists  *}
  Found       : Boolean;             {* Boolean checking value             *}
  AuthorNames : AuthorArray;         {* Arrar of author names              *}

BEGIN {* CheckBoth *}

  {* Gets the element to delete *}
  SearchNode.Key := SearchBook.BookNode.Title;
  RetrieveTreeElement (Trees[Title],SearchNode,Found);

  {* Checks to see if it exists, if so act upon it, otherwise NotFound *}
  IF Found THEN
      BEGIN {* If *}
        Found := False;
        ResetList (SearchNode.BookList);
        WHILE (SearchNode.Booklist.CurrentItem <> NIL) AND NOT(Found) DO
          BEGIN {* While *}
            Found := Match (SearchNode,SearchBook,CheckBook,AuthorNames);
            IF Found THEN
                BEGIN {* If *}
                  CASE Process OF
                    PrintOut : OutputBookElement (SearchNode,Both,CheckBook.BookNode.Authors,LogFile);
                    Remove   : DeleteBook (SearchNode,Both,CheckBook.BookNode.Authors,BookList,Trees,LogFile);
                  END; {* Case *}
                END; {* If *}
          END; {* While *}
        IF Not(Found) THEN NotFound (LogFile);
      END {* If *}
    ELSE NotFound (LogFile);

END; {* CheckBoth *}

{***************************************************************************}
PROCEDURE CheckTitle
  (    SearchBook : ListElementType; {* Search book entered by user        *}
   VAR BookList   : ListType;        {* Master list of books               *}
   VAR Trees      : IndexTrees;      {* Trees to be checked for values     *}
       Process    : ProcessMethod;   {* What process to perform            *}
   VAR LogFile    : Text);           {* LogFile                            *}
{* Purpse : To search for a book via title                                 *}
{* Pre    : Position of search book unknown                                *}
{* Post   : Position (if it exists) is known                               *}

VAR
  SearchNode : TreeElementType;      {* Node to be searched for            *}
  Found      : Boolean;              {* Boolean checking value             *}

BEGIN {* CheckTitle *}

  {* Gets the element to delete *}
  SearchNode.Key := SearchBook.BookNode.Title;
  RetrieveTreeElement (Trees[Title],SearchNode,Found);

  {* Checks to see if it exists, if so act upon it otherwise NotFound *}
  IF Found THEN
      CASE Process OF
        PrintOut : OutputBookElement (SearchNode,One,'',LogFile);
        Remove   : DeleteBook (SearchNode,One,'',BookList,Trees,LogFile);
      END {* Case *}
    ELSE NotFound (LogFile);

END; {* CheckTitle *}

{***************************************************************************}
FUNCTION Included
  (VAR SearchNode  : TreeElementType;{* Node to be examined                *}
   VAR AuthorNames : AuthorArray) : Boolean;{* All author names in node    *}
{* Purpse : To find if all authors included in the node                    *}
{* Pre    : Matches and all included names unknown                         *}
{* Post   : Includes known and boolean sent back to main call              *}

VAR
  TempInclude : Boolean;             {* Boolean checking value             *}
  Count       : Integer;             {* Loop control variable              *}

BEGIN {* Included *}

  {* Check for more than one item in the list *}
  IF SearchNode.BookList.CurrentItem = NIL THEN
      TempInclude := True
    ELSE TempInclude := False;

  {* Checks all nodes for matchin authors to see if they are included *}
  WHILE (SearchNode.BookList.CurrentItem <> NIL) AND NOT(TempInclude) DO
    BEGIN {* While *}
      Count := 2;
      TempInclude := True;
      WHILE (AuthorNames[Count] <> '') AND TempInclude DO
        BEGIN {* While *}
          IF pos(AuthorNames[Count],SearchNode.BookList.CurrentItem^.Info.BookNode.Authors) = 0 THEN
              TempInclude := False;
          inc(Count);
        END; {* While *}

      {* Move on to the next one *}
      SearchNode.BookList.CurrentItem := SearchNode.BookList.CurrentItem^.Next;
    END; {* While *}

  {* Return the result *}
  Included := TempInclude;

END; {* Included *}

{***************************************************************************}
PROCEDURE CheckAuthor
  (    SearchBook : ListElementType; {* Search book entered by user        *}
   VAR BookList   : ListType;        {* Master list of books               *}
   VAR Trees      : IndexTrees;      {* Trees to be checked for values     *}
       Process    : ProcessMethod;   {* What process to perform            *}
   VAR LogFile    : Text);           {* LogFile                            *}
{* Purpse : To search for a book via author                                *}
{* Pre    : Position of search book unknown                                *}
{* Post   : Position (if it exists) is known                               *}

VAR
  SearchNode  : TreeElementType;     {* Node to be searched for            *}
  Found       : Boolean;             {* Boolean checking value             *}
  Count       : Integer;             {* Loop control variable              *}
  AuthorNames : AuthorArray;         {* Array of author names              *}

BEGIN {* CheckAuthor *}

  {* Find out the authors *}
  BreakUpAuthors (SearchBook.BookNode.Authors,AuthorNames);
  Count := 1;
  SearchNode.Key := AuthorNames[Count];
  RetrieveTreeElement (Trees[Author],SearchNode,Found);
  ResetList (SearchNode.BookList);
  IF Found THEN
      BEGIN {* If *}

        {* Check to see if they all exit, and act upon the result *}
        IF Included (SearchNode,AuthorNames) THEN
            CASE Process OF
              PrintOut : OutputBookElement (SearchNode,One,'',LogFile);
              Remove   : DeleteBook (SearchNode,One,'',BookList,Trees,LogFile);
            END {* Case *}
          ELSE NotFound (LogFile);
      END {* If *}
    ELSE NotFound (LogFile);

END; {* CheckAuthor *}

{***************************************************************************}

END. {* Advanced *}