PROGRAM ImgEditor;

USES
  NewDelay,
  Crt,
  Dos,
  Menus,
  Utility,
  SndData,
  ImgData,
  GrubData;

CONST
  ExitVals_   = [#27,#13];
  Default_    = ' ';
  NilChar_    = #0;

  {* Requires a NilChar before recognizing these numbers *}
  Up_         = #72;
  Right_      = #77;
  Down_       = #80;
  Left_       = #75;
  AltUp_      = #71;
  AltRight_   = #81;
  AltDown_    = #79;
  AltLeft_    = #83;
  Help_       = #59;
  Save_       = #60;
  Load_       = #61;
  Export_     = #62;
  Size_       = #63;
  BrushChr_   = #64;
  BrushNum_   = #65;
  Fill_       = #66;
  Canvas_     = #67;
  Root_       = #68;
  Ascii_      = #13;

  {* Must be first character entered *}
  Exit_       = #27;
  ImageName_  = #8;
  ShowInfo_   = #92;
  PlacePixel_ = #32;
  BackBlack_  = #49;
  BackBlue_   = #50;
  BackGreen_  = #51;
  BackTeal_   = #52;
  BackRed_    = #53;
  BackPurple_ = #54;
  BackOrange_ = #55;
  BackWhite_  = #56;
  ForeBlack_  = #113;
  ForeBlue_   = #119;
  ForeGreen_  = #101;
  ForeTeal_   = #114;
  ForeRed_    = #116;
  ForePurple_ = #121;
  ForeOrange_ = #117;
  ForeWhite_  = #105;


TYPE
  CurrentType = RECORD
                  Pixel  : PixelType;
                  XCoord,
                  YCoord : Byte;
                END;

VAR
  Width,
  Height,
  Canvas,
  StartX,
  StartY    : Byte;
  Image     : ARRAY [1..80,1..50] OF PixelType;
  ImageFile : FILE;
  ImageName : FileName;
  Finished  : Boolean;
  Root      : FileName;
  OrigMode  : Integer;
  DirInfo   : SearchRec;

{****************************************************************************}
PROCEDURE UpdateOffsets;

BEGIN
  StartX := (80 - Width) DIV 2;
  StartY := (50 - Height) DIV 2;
END;

{****************************************************************************}
PROCEDURE RefreshScreen (Current : CurrentType);

VAR
  Count,
  Counter   : Byte;
  TempPixel : PixelType;

BEGIN
  TempPixel.Character := Default_;
  TempPixel.Color := (Canvas * $10) + Canvas;
  FOR Count := 1 TO 50 DO
    FOR Counter := 1 TO 80 DO
      DrawPixel (TempPixel,Counter,Count);
  FOR Count := 1 TO Height DO
    FOR Counter := 1 TO Width DO
      DrawPixel (Image[Counter,Count],Counter + StartX,Count + StartY);
  DrawPixel (Current.Pixel,Current.XCoord + StartX,Current.YCoord + StartY);
END;

{****************************************************************************}
PROCEDURE InitializeEditor (VAR Current : CurrentType);

VAR
  Count,
  Counter   : Byte;
  TempInt,
  NumRead   : Integer;
  InitFile  : Text;
  TempStr   : STRING;

BEGIN
  Root := GetRoot;
  ImageName := '';
  IF ParamStr(1) <> '' THEN
      BEGIN
        ImageName := ParamStr(1);
        IF ParamStr(2) <> '' THEN
            val(ParamStr(2),Canvas,TempInt)
          ELSE
            Canvas := 0;
      END;
  IF ImageName = '' THEN
    BEGIN
      findfirst ('root.txt',Archive,DirInfo);
      IF DosError = 0 THEN
          BEGIN
            assign (InitFile,'root.txt');
            reset (InitFile);
            readln (InitFile,Root);
            readln (InitFile,ImageName);
            readln (InitFile,Canvas);
            close (InitFile);
          END;
    END;
  IF ImageName = '' THEN
    BEGIN
      ImageName := 'new';
      Canvas := 0;
    END;
  Width := 80;
  Height := 50;
  FOR Count := 1 TO Height DO
    FOR Counter := 1 TO Width DO
      BEGIN
        Image[Counter,Count].Character := Default_;
        Image[Counter,Count].Color := (Canvas * $10) + Canvas;
      END;
  findfirst (Root + '\images\' + ImageName + ImageExtention,Archive,DirInfo);
  IF DosError = 0 THEN
      BEGIN
        NumRead := 0;
        assign (ImageFile,Root + '\images\' + ImageName + ImageExtention);
        reset (ImageFile,1);
        blockread (ImageFile, Width, sizeof(Width), NumRead);
        blockread (ImageFile, Height, sizeof(Height), NumRead);
        UpdateOffsets;
        FOR Count := 1 TO Height DO
          FOR Counter := 1 TO Width DO
            blockread (ImageFile, Image[Counter,Count],
                       sizeof(Image[Counter,Count]), NumRead);
        close (ImageFile);
      END;
  Current.Pixel := Image[1,1];
  Current.XCoord := 1;
  Current.YCoord := 1;
END;

{****************************************************************************}
PROCEDURE ShutDownEditor;

VAR
  InitFile  : Text;

BEGIN
  findfirst ('root.txt',Archive,DirInfo);
  IF DosError = 0 THEN
      BEGIN
        assign (InitFile,'root.txt');
        rewrite (InitFile);
        writeln (InitFile,Root);
        writeln (InitFile,ImageName);
        writeln (InitFile,Canvas);
        close (InitFile);
      END;
  Finished := True;
END;

{****************************************************************************}
PROCEDURE DisplayHelp (Current : CurrentType);

VAR
  TempChar : Char;

BEGIN
  MakeWin (7,0,1,1,80,50,False,False);
  WrtS (1,1,'SPACEBAR - places the color/character in that current location');
  WrtS (1,2,'ARROWS - move the curser around the image');
  WrtS (1,3,'HOME, DELETE, END, PAGE DOWN - keep drawing with the current selection');
  WrtS (1,4,'F1 - display help file (command list)');
  WrtS (1,5,'F2 - save current image');
  WrtS (1,6,'F3 - load an image');
  WrtS (1,7,'F4 - export to image text file');
  WrtS (1,8,'F5 - enter new dimensions for image');
  WrtS (1,9,'F6 - type in a new ascii character');
  WrtS (1,10,'F7 - type a number (0-255) for an ascii character');
  WrtS (1,11,'F8 - fill the entire work space with background color');
  WrtS (1,12,'F9 - change the canvas to the background color');
  WrtS (1,13,'F10 - change the Root directory to work from');
  WrtS (1,14,'BACKSPACE - change the name of the current file being worked upon');
  WrtS (1,15,'\ - view the current image''s properites');
  WrtS (1,16,'ENTER - view ASCII chart (usable characters)');
  WrtS (1,17,'12345678 - changes the foreground color');
  WrtS (1,18,'QWERTYUI - changes the background color');
  WrtS (1,19,'ESC - exits the program');

  WrtS (1,21,'If the color matches what is currently selected it will switch to its');
  WrtS (1,22,'alternate high or low intesity, always starts low intensity');

  WrtS (1,24,'Command line parameter tells what the name of the image to edit is. If it');
  WrtS (1,25,'finds it in the dir it will load that one, otherwise start blank');

  WrtS (1,27,'Cursor is in the state of the current "brush" selection');

  WrtS (1,29,'Default character is Space (#32)');

  WrtS (1,31,'If you change the dimensions of the image to larger than what is current,');
  WrtS (1,32,'it will add background to the surrounding border to the right and down');

  WrtS (1,34,'If you change the dimensions of the image to smaller than what is current,');
  WrtS (1,35,'it will cut off the right and bottom sides');

  WrtS (1,37,'Display is always started at 1,1 coords');

  TcTb (14,0);
  WrtS (1,50,'Press <ENTER> or <ESCAPE> to return to the editor.');
  REPEAT
    TempChar := readkey;
  UNTIL TempChar IN ExitVals_;
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE DisplayInfo (Current : CurrentType);

VAR
  TempPixel : PixelType;
  TempChar  : Char;

BEGIN
  MakeWin (7,0,1,1,80,50,False,False);
  gotoXY (1,1);
  writeln ('Game: ', Root);
  writeln;
  writeln ('Image Name: ', ImageName);
  writeln ('Image Size: ', Width, 'x', Height);
  writeln;
  writeln ('Current Brush: ');
  writeln ('Current (X,Y): (', Current.XCoord, ',', Current.YCoord, ')');
  writeln;
  writeln ('Canvas Color: ');
  DrawPixel (Current.Pixel,16,6);

  TempPixel.Character := ' ';
  TempPixel.Color := Canvas;
  DrawPixel (TempPixel,15,9);

  TcTb (14,0);
  WrtS (1,50,'Press <ENTER> or <ESCAPE> to return to the editor.');
  REPEAT
    TempChar := readkey;
  UNTIL TempChar IN ExitVals_;
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE AsciiChart (Current : CurrentType);

VAR
  Count,
  Counter   : Byte;
  TempChar  : Char;

BEGIN
  MakeWin (7,0,1,1,80,50,False,False);
  gotoXY (1,1);
  TcTb (8,0);
  WrtS (1,1,'     1=      2=      3=      4=      5=      6=      9=     11=     12=     14=');
  WrtS (1,3,'    15=     16=     17=     18=     19=     20=');

  TcTb (7,0);
  WrtC (8,1,chr(1));
  WrtC (16,1,chr(2));
  WrtC (24,1,chr(3));
  WrtC (32,1,chr(4));
  WrtC (40,1,chr(5));
  WrtC (48,1,chr(6));
  WrtC (56,1,chr(9));
  WrtC (64,1,chr(11));
  WrtC (72,1,chr(12));
  WrtC (80,1,chr(14));
  WrtC (8,3,chr(15));
  WrtC (16,3,chr(16));
  WrtC (24,3,chr(17));
  WrtC (32,3,chr(18));
  WrtC (40,3,chr(19));
  WrtC (48,3,chr(20));

  gotoXY (49,3);
  FOR Count := 21 TO 254 DO
    BEGIN
      TcTb (8,0);
      write (Count:6, '=');
      TcTb (7,0);
      write (chr(Count));
      IF (Count MOD 10 = 4) AND (Count <> 254) THEN writeln;
    END;

  TcTb (14,0);
  WrtS (1,50,'Press <ENTER> or <ESCAPE> to return to the editor.');
  REPEAT
    TempChar := readkey;
  UNTIL TempChar IN ExitVals_;
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE ChangeCanvas (Current : CurrentType);

BEGIN
  Canvas := Current.Pixel.Color DIV $10;
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE NewForeColor (VAR Current : CurrentType; NewColor : Byte);

BEGIN
  IF (Current.Pixel.Color DIV $10) = NewColor THEN
      BEGIN
        IF (Current.Pixel.Color DIV $10) > 7 THEN
            Current.Pixel.Color := (Current.Pixel.Color MOD $10) + ((NewColor - 8) * $10)
          ELSE
            Current.Pixel.Color := (Current.Pixel.Color MOD $10) + ((NewColor + 8) * $10)
      END
    ELSE Current.Pixel.Color := (Current.Pixel.Color MOD $10) + (NewColor * $10);
  DrawPixel (Current.Pixel,Current.XCoord + StartX,Current.YCoord + StartY);
END;

{****************************************************************************}
PROCEDURE NewBackColor (VAR Current : CurrentType; NewColor : Byte);

BEGIN
  IF (Current.Pixel.Color MOD $10) = NewColor THEN
      BEGIN
        IF (Current.Pixel.Color MOD $10) > 7 THEN
            Current.Pixel.Color := Current.Pixel.Color - (Current.Pixel.Color MOD $10) + (NewColor - 8)
          ELSE
            Current.Pixel.Color := Current.Pixel.Color - (Current.Pixel.Color MOD $10) + (NewColor + 8)
      END
    ELSE Current.Pixel.Color := Current.Pixel.Color - (Current.Pixel.Color MOD $10) + NewColor;
  DrawPixel (Current.Pixel,Current.XCoord + StartX,Current.YCoord + StartY);
END;

{****************************************************************************}
PROCEDURE EnterNewChar (VAR Current : CurrentType);

BEGIN
  MakeWin (7,0,1,1,80,50,False,False);
  gotoXY (1,1);
  write ('Type new chracter and then press <ENTER>: ');
  readln (Current.Pixel.Character);
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE EnterNewCharNum (VAR Current : CurrentType);

VAR
  TempByte : Byte;

BEGIN
  MakeWin (7,0,1,1,80,50,False,False);
  gotoXY (1,1);
  write ('Type new chracter number and then press <ENTER>: ');
  readln (TempByte);
  Current.Pixel.Character := chr(TempByte);
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE MoveUp (VAR Current : CurrentType; Clear : Boolean);

BEGIN
  IF NOT(Clear) THEN
    Image[Current.XCoord,Current.YCoord] := Current.Pixel;
  IF Current.YCoord > 1 THEN
    BEGIN
      DrawPixel (Current.Pixel,Current.XCoord + StartX,Current.YCoord + StartY - 1);
      DrawPixel (Image[Current.XCoord,Current.YCoord],Current.XCoord + StartX,Current.YCoord + StartY);
      dec(Current.YCoord);
    END;
END;

{****************************************************************************}
PROCEDURE MoveRight (VAR Current : CurrentType; Clear : Boolean);

BEGIN
  IF NOT(Clear) THEN
    Image[Current.XCoord,Current.YCoord] := Current.Pixel;
  IF Current.XCoord < Width THEN
    BEGIN
      DrawPixel (Current.Pixel,Current.XCoord + StartX + 1,Current.YCoord + StartY);
      DrawPixel (Image[Current.XCoord,Current.YCoord],Current.XCoord + StartX,Current.YCoord + StartY);
      inc(Current.XCoord);
    END;
END;

{****************************************************************************}
PROCEDURE MoveDown (VAR Current : CurrentType; Clear : Boolean);

BEGIN
  IF NOT(Clear) THEN
    Image[Current.XCoord,Current.YCoord] := Current.Pixel;
  IF Current.YCoord < Height THEN
    BEGIN
      DrawPixel (Current.Pixel,Current.XCoord + StartX,Current.YCoord + StartY + 1);
      DrawPixel (Image[Current.XCoord,Current.YCoord],Current.XCoord + StartX,Current.YCoord + StartY);
      inc(Current.YCoord);
    END;
END;

{****************************************************************************}
PROCEDURE MoveLeft (VAR Current : CurrentType; Clear : Boolean);

BEGIN
  IF NOT(Clear) THEN
    Image[Current.XCoord,Current.YCoord] := Current.Pixel;
  IF Current.XCoord > 1 THEN
    BEGIN
      DrawPixel (Current.Pixel,Current.XCoord + StartX - 1,Current.YCoord + StartY);
      DrawPixel (Image[Current.XCoord,Current.YCoord],Current.XCoord + StartX,Current.YCoord + StartY);
      dec(Current.XCoord);
    END;
END;

{****************************************************************************}
PROCEDURE PlacePixel (Current : CurrentType);

BEGIN
  Image[Current.XCoord,Current.YCoord] := Current.Pixel;
END;

{****************************************************************************}
PROCEDURE FillImage (Current : CurrentType);

VAR
  Count,
  Counter : Byte;

BEGIN
  FOR Count := 1 TO Height DO
    FOR Counter := 1 TO Width DO
      BEGIN
        Image[Counter,Count].Character := Current.Pixel.Character;
        Image[Counter,Count].Color := Current.Pixel.Color;
      END;
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE ChangeSize (VAR Current : CurrentType);

VAR
  Count,
  Counter,
  NewWidth,
  NewHeight : Byte;

BEGIN
  MakeWin (7,0,1,1,80,50,False,False);
  gotoXY (1,1);
  write ('Type the new Width and press <ENTER>: ');
  readln (NewWidth);
  write ('Type the new Height and press <ENTER>: ');
  readln (NewHeight);

  IF NewWidth > Width THEN
      BEGIN
        FOR Count := 1 TO Height DO
          FOR Counter := (Width + 1) TO NewWidth DO
            BEGIN
              Image[Counter,Count].Character := Current.Pixel.Character;
              Image[Counter,Count].Color := Current.Pixel.Color;
            END;
      END;
  Width := NewWidth;
  IF NewHeight > Height THEN
      BEGIN
        FOR Count := Height + 1 TO NewHeight DO
          FOR Counter := 1 TO Width DO
            BEGIN
              Image[Counter,Count].Character := Current.Pixel.Character;
              Image[Counter,Count].Color := Current.Pixel.Color;
            END;
      END;
  Height := NewHeight;

  IF Current.XCoord > Width THEN Current.XCoord := Width;
  IF Current.YCoord > Height THEN Current.YCoord := Height;

  UpdateOffsets;
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE ChangeFileName (Current : CurrentType);

BEGIN
  MakeWin (7,0,1,1,80,50,False,False);
  gotoXY (1,1);
  write ('Type the new filename and then press <ENTER>: ');
  readln (ImageName);
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE ChangeRoot (Current : CurrentType);

BEGIN
  MakeWin (7,0,1,1,80,50,False,False);
  gotoXY (1,1);
  write ('Type the root game directory and then press <ENTER>: ');
  readln (Root);
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE SaveImage (Current : CurrentType);

VAR
  Count,
  Counter    : Byte;
  NumWritten : Integer;
  TempChar   : Char;

BEGIN
  NumWritten := 0;
  assign (ImageFile,Root + '\images\' + ImageName + ImageExtention);
  rewrite (ImageFile,1);
  blockwrite (ImageFile, Width, sizeof(Width), NumWritten);
  blockwrite (ImageFile, Height, sizeof(Height), NumWritten);
  FOR Count := 1 TO Height DO
    FOR Counter := 1 TO Width DO
      blockwrite (ImageFile, Image[Counter,Count],
                 sizeof(Image[Counter,Count]), NumWritten);
  close (ImageFile);
  MakeWin (7,0,1,1,80,50,False,False);
  gotoXY (1,1);
  write (ImageName, ' saved.');
  TcTb (14,0);
  WrtS (1,50,'Press <ENTER> or <ESCAPE> to return to the editor.');
  REPEAT
    TempChar := readkey;
  UNTIL TempChar IN ExitVals_;
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE LoadImage (VAR Current : CurrentType);

VAR
  Count,
  Counter  : Byte;
  NumRead  : Integer;
  TempChar : Char;

BEGIN

  MakeWin (7,0,1,1,80,50,False,False);
  gotoXY (1,1);
  write ('Type name of image to load: ');
  readln (ImageName);

  Width := 80;
  Height := 50;
  FOR Count := 1 TO Height DO
    FOR Counter := 1 TO Width DO
      BEGIN
        Image[Counter,Count].Character := Default_;
        Image[Counter,Count].Color := (Canvas * $10) + Canvas;
      END;
  findfirst (Root + '\images\' + ImageName + ImageExtention,Archive,DirInfo);
  IF DosError = 0 THEN
      BEGIN
        NumRead := 0;
        assign (ImageFile,Root + '\images\' + ImageName + ImageExtention);
        reset (ImageFile,1);
        blockread (ImageFile, Width, sizeof(Width), NumRead);
        blockread (ImageFile, Height, sizeof(Height), NumRead);
        UpdateOffsets;
        FOR Count := 1 TO Height DO
          FOR Counter := 1 TO Width DO
            blockread (ImageFile, Image[Counter,Count],
                       sizeof(Image[Counter,Count]), NumRead);
        close (ImageFile);
      END;
  Current.Pixel := Image[1,1];
  Current.XCoord := 1;
  Current.YCoord := 1;

  MakeWin (7,0,1,1,80,50,False,False);
  gotoXY (1,1);
  write (ImageName, ' loaded.');
  TcTb (14,0);
  WrtS (1,50,'Press <ENTER> or <ESCAPE> to return to the editor.');
  REPEAT
    TempChar := readkey;
  UNTIL TempChar IN ExitVals_;
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE ExportImage (Current : CurrentType);

VAR
  Count,
  Counter    : Byte;
  TempChar   : Char;
  ExportFile : Text;

BEGIN
  assign (ExportFile,Root + '\images\' + ImageName + '.txt');
  rewrite (ExportFile);
  writeln (ExportFile, Width);
  writeln (ExportFile, Height);
  FOR Count := 1 TO Height DO
    BEGIN
      FOR Counter := 1 TO Width DO
        write (ExportFile, Image[Counter,Count].Character, ' ', Image[Counter,Count].Color, ' ');
      writeln (ExportFile);
    END;
  close (ExportFile);
  MakeWin (7,0,1,1,80,50,False,False);
  gotoXY (1,1);
  write (ImageName, ' exported.');
  TcTb (14,0);
  WrtS (1,50,'Press <ENTER> or <ESCAPE> to return to the editor.');
  REPEAT
    TempChar := readkey;
  UNTIL TempChar IN ExitVals_;
  RefreshScreen (Current);
END;

{****************************************************************************}
PROCEDURE ExecuteCommand (Command : Char; VAR Current : CurrentType);

BEGIN
  {* All input interfaces need to be improved and made more user friendly *}
  CASE Command OF
    #81 : Command := #113;
    #87 : Command := #119;
    #69 : Command := #101;
    #82 : Command := #114;
    #84 : Command := #116;
    #89 : Command := #121;
    #85 : Command := #117;
    #73 : Command := #105;
  END;

  CASE Command OF
    NilChar_ : BEGIN
                 Command := readkey;
                 CASE Command OF
                   Up_       : MoveUp (Current,True);
                   Right_    : MoveRight (Current,True);
                   Down_     : MoveDown (Current,True);
                   Left_     : MoveLeft (Current,True);
                   AltUp_    : MoveUp (Current,False);
                   AltRight_ : MoveRight (Current,False);
                   AltDown_  : MoveDown (Current,False);
                   AltLeft_  : MoveLeft (Current,False);
                   Help_     : DisplayHelp (Current);
                   Save_     : SaveImage (Current);
                   Load_     : LoadImage (Current);
                   Export_   : ExportImage (Current);
                   Size_     : ChangeSize (Current);
                   BrushChr_ : EnterNewChar (Current);
                   BrushNum_ : EnterNewCharNum (Current);
                   Fill_     : FillImage (Current);
                   Canvas_   : ChangeCanvas (Current);
                   Root_     : ChangeRoot (Current);
                 END;
               END;
    ImageName_  : ChangeFileName (Current);
    ShowInfo_   : DisplayInfo (Current);
    Ascii_      : AsciiChart (Current);
    Exit_       : ShutDownEditor;
    PlacePixel_ : PlacePixel (Current);
    ForeBlack_  : NewForeColor (Current,0);
    ForeBlue_   : NewForeColor (Current,1);
    ForeGreen_  : NewForeColor (Current,2);
    ForeTeal_   : NewForeColor (Current,3);
    ForeRed_    : NewForeColor (Current,4);
    ForePurple_ : NewForeColor (Current,5);
    ForeOrange_ : NewForeColor (Current,6);
    ForeWhite_  : NewForeColor (Current,7);
    BackBlack_  : NewBackColor (Current,0);
    BackBlue_   : NewBackColor (Current,1);
    BackGreen_  : NewBackColor (Current,2);
    BackTeal_   : NewBackColor (Current,3);
    BackRed_    : NewBackColor (Current,4);
    BackPurple_ : NewBackColor (Current,5);
    BackOrange_ : NewBackColor (Current,6);
    BackWhite_  : NewBackColor (Current,7);
  END;
END;

{****************************************************************************}
PROCEDURE ControlProcessing;

VAR
  Command   : Char;
  Current   : CurrentType;

BEGIN
  InitializeEditor (Current);
  RefreshScreen (Current);

  REPEAT
    Command := Readkey;
    ExecuteCommand (Command,Current);
  UNTIL Finished;

END;

{****************************************************************************}

BEGIN
  OrigMode := LastMode;
  TextMode (C80 + Font8x8);
  HighIntensity;
  CursorOff;
  ControlProcessing;
  CursorOn;
  TextMode (OrigMode);
END.