UNIT Menus;

INTERFACE

USES
  NewDelay,
  Crt,
  Utility;

CONST
  MaxMenu = 4;

TYPE
  ExitCharSet = SET OF Char;
  MenuElementType = RECORD
                      Position    : Integer;
                      Title       : STRING[70];
                      FileName    : STRING[12];
                      Launcher    : STRING[12];
                      Version     : LongInt;
                      Activated   : Boolean;
                      LongDescrpt : ARRAY [1..3] OF STRING[70];
                    END;

  MenuRelationType = (Less, Greater, Equal);

  MenuPointerType = ^MenuNodeType;
  MenuNodeType = RECORD
    Info   : MenuElementType;
    Next   : MenuPointerType
  END;
  MenuList = RECORD
    Length       : Integer;
    CurrentItem  : MenuPointerType;
    ListData     : MenuPointerType
  END;
  MenuType = ARRAY [1..MaxMenu] OF STRING;

FUNCTION MenuEmptyList (List : MenuList) : Boolean;
FUNCTION MenuFullList (List : MenuList) : Boolean;
FUNCTION MenuListLength (List : MenuList) : Integer;
PROCEDURE MenuCreateList (VAR List : MenuList);
PROCEDURE MenuDestroyList (VAR List : MenuList);
PROCEDURE MenuRetrieveElement (List : MenuList; VAR Element : MenuElementType);
PROCEDURE MenuInsertElement (VAR List : MenuList; NewElement : MenuElementType);
PROCEDURE MenuDeleteElement (VAR List : MenuList; Element  : MenuElementType);
PROCEDURE MenuResetList (VAR List : MenuList);
PROCEDURE MenuGetNextItem (VAR List : MenuList; VAR Element  : MenuElementType);
PROCEDURE ArraySelect
  (RegText, RegBack, SelText, SelBack, Startx, StartY, Height : Byte;
   VAR Cursor : Byte; VAR Choice : Char; Menu : MenuType; ExitVals : ExitCharSet);

IMPLEMENTATION

{***************************************************************************}
FUNCTION CompareName (Item1, Item2 : MenuElementType) : MenuRelationType;

BEGIN
  IF Item1.Title < Item2.Title THEN
      CompareName := Less
    ELSE IF Item1.Title > Item2.Title THEN
      CompareName := Greater
    ELSE CompareName := Equal;
END;

{***************************************************************************}
FUNCTION ComparePos (Item1, Item2 : MenuElementType) : MenuRelationType;

BEGIN
  IF Item1.Position < Item2.Position THEN
      ComparePos := Less
    ELSE IF Item1.Position > Item2.Position THEN
      ComparePos := Greater
    ELSE ComparePos := Equal;
END;

(*************************************************************)
FUNCTION MenuEmptyList (List : MenuList) : Boolean;

BEGIN
  MenuEmptyList := (List.ListData =  NIL)
END;

(*************************************************************)
FUNCTION MenuFullList (List : MenuList) : Boolean;

BEGIN
  MenuFullList := False
END;

(*************************************************************)
FUNCTION MenuListLength (List : MenuList) : Integer;

BEGIN
  MenuListLength := List.Length
END;

(*************************************************************)
PROCEDURE MenuCreateList (VAR List : MenuList);

BEGIN
  List.ListData := NIL;
  List.Length := 0
END;

(*************************************************************)
PROCEDURE MenuDestroyList (VAR List : MenuList);

VAR
  TempPtr : MenuPointerType;

BEGIN
  WITH List DO
    BEGIN
      List.Length := 0;
      WHILE ListData <>  NIL DO
        BEGIN
          TempPtr := ListData;
          ListData := ListData^.Next;
          Dispose (TempPtr)
        END
      END
END;

(*************************************************************)
PROCEDURE MenuRetrieveElement (List : MenuList; VAR Element : MenuElementType);

VAR
  Location     : MenuPointerType;

BEGIN
  Location := List.ListData;
  WHILE Location^.Info.Position <> Element.Position DO
    Location := Location^.Next;
  Element := Location^.Info;
END;

(*************************************************************)
PROCEDURE MenuInsertElement (VAR List : MenuList; NewElement : MenuElementType);

VAR
  NewNode      : MenuPointerType;
  PredLoc      : MenuPointerType;
  Location     : MenuPointerType;
  MoreToSearch : Boolean;

BEGIN
  WITH List DO
    BEGIN
      Location := List.ListData;
      PredLoc := NIL;
      MoreToSearch := Location <> NIL;
      List.Length := List.Length + 1;
      WHILE MoreToSearch DO
        CASE CompareName (NewElement, Location^.Info) OF
          Greater : BEGIN
                      PredLoc := Location;
                      Location := Location^.Next;
                      MoreToSearch := Location <> NIL
                    END;
          Less    : MoreToSearch := False
        END;
      New(NewNode);
      NewNode^.Info := NewElement;
      IF PredLoc = NIL
        THEN
          BEGIN
            NewNode^.Next := ListData;
            ListData := NewNode
          END
        ELSE
          BEGIN
            NewNode^.Next := PredLoc^.Next;
            PredLoc^.Next := NewNode
          END
    END
END;

(*************************************************************)
PROCEDURE MenuDeleteElement (VAR List : MenuList; Element : MenuElementType);

VAR
  PredLoc   : MenuPointerType;
  Location  : MenuPointerType;

  TempPtr   : MenuPointerType;
  MoreToSearch : Boolean;

BEGIN
  WITH List DO
    BEGIN
      Location := ListData;
      PredLoc := NIL;
      MoreToSearch := Location <> NIL;
      Length := Length - 1;
      WHILE MoreToSearch DO
        CASE CompareName (Element, Location^.Info) OF
          Greater: BEGIN
                     PredLoc := Location;
                     Location := Location^.Next;
                   END;
          Equal  : MoreToSearch := False
        END;
      IF PredLoc = NIL
        THEN
          BEGIN
            TempPtr := Location;
            ListData := Location^.Next;
          END
        ELSE
          BEGIN
            TempPtr := Location;
            PredLoc^.Next := Location^.Next
          END;
      Dispose (TempPtr)
    END
END;

(*************************************************************)
PROCEDURE MenuResetList (VAR List : MenuList);

BEGIN
  List.CurrentItem := List.ListData
END;

(***********************************************************)
PROCEDURE MenuGetNextItem (VAR List : MenuList; VAR Element : MenuElementType);

BEGIN
  Element := List.CurrentItem^.Info;
  List.CurrentItem := List.CurrentItem^.Next
END;

{***************************************************************************}
PROCEDURE ArraySelect
  (RegText, RegBack, SelText, SelBack, Startx, StartY, Height : Byte;
   VAR Cursor : Byte; VAR Choice : Char; Menu : MenuType; ExitVals : ExitCharSet);

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}
PROCEDURE Move
  (Choice : char;
   VAR Cursor : byte);

BEGIN
  TcTb (RegText, RegBack);
  WrtS (StartX, (Cursor + StartY - 1), Menu[Cursor]);
  CASE Choice OF
    #72 : IF Cursor > 1 THEN dec(Cursor);
    #80 : IF Cursor < Height THEN inc(Cursor);
  END;
  TcTb (SelText, SelBack);
  WrtS (StartX, (Cursor + StartY - 1), Menu[Cursor]);
END;

{+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++}

BEGIN
  TcTb (SelText, SelBack);
  WrtS (StartX, (Cursor + StartY - 1), Menu[Cursor]);
  Choice := readkey;
  WHILE NOT(Choice IN ExitVals) DO
    BEGIN
      IF ((Choice = #72) AND (Cursor > 1)) OR ((Choice = #80) AND (Cursor < Height)) THEN
        BEGIN
          Move (Choice,Cursor);
        END;
      Choice := readkey;
    END;
END;

(***********************************************************)

END.




