UNIT Utility;

INTERFACE

USES
  DOS,
  NewDelay,
  Crt;

TYPE
  StringArray = ARRAY [1..60] OF Char;
  LargeBufType = ARRAY [1..64000] OF Char;

FUNCTION  Convert (Strng : STRING; VAR Int : Integer) : Boolean;
FUNCTION  ConvertReal (Strng : STRING; VAR Reel : Real): Boolean;
PROCEDURE HighIntensity;
PROCEDURE Beep (Pitch, Time : Integer);
PROCEDURE TcTb (Tc, Tb : byte);
PROCEDURE WrtC (X, Y : byte; Chr : char);
PROCEDURE WrtS (X, Y : byte; Str : STRING);
PROCEDURE WrtI (X, Y : byte; Int : integer);
PROCEDURE WrtR (X, Y : byte; Reel : real; Field, Decimal : byte);
PROCEDURE FrmtI (X, Y : byte; Int : integer; Field : byte);
PROCEDURE Clear (Text, Backcolor, Ulx, Uly, Lrx, Lry : byte);
PROCEDURE CursorOn;
PROCEDURE CursorOff;
PROCEDURE MakeWin (TC, BC, Ulx, Uly, Width, Height : byte; Border, Shadow : Boolean);
PROCEDURE CopyFile (FileName, Source, Destination : STRING);
PROCEDURE MakeDir (NewDir : STRING);

IMPLEMENTATION

{**************************************************************************}
FUNCTION Convert (Strng : STRING; VAR Int : Integer) : Boolean;

VAR
  ErrorPos : Integer;

BEGIN
  val(Strng, Int, ErrorPos);
  IF ErrorPos = 0 THEN
      Convert := True
    ELSE Convert := False;
END;

{**************************************************************************}
FUNCTION ConvertReal (Strng : STRING; VAR Reel : Real) : Boolean;

VAR
  WholeStr,
  DecimalStr : STRING;
  Whole,
  Decimal    : Integer;
  Long,
  ErrorPos,
  DecimalPos : Byte;

BEGIN
  WholeStr := Strng;
  DecimalStr := Strng;
  Long := Length (Strng);
  DecimalPos := pos ('.',Strng);
  IF DecimalPos <> 0 THEN
      BEGIN
        Delete (WholeStr,DecimalPos,Long);
        Delete (DecimalStr,1,DecimalPos);
      END
    ELSE
      BEGIN
        WholeStr := Strng;
        DecimalStr := '00';
      END;

  IF NOT (Convert (WholeStr,Whole) AND Convert (DecimalStr,Decimal)) THEN
      ConvertReal := False
    ELSE
      BEGIN
        Reel := (Whole + Decimal/100);
        ConvertReal := True;
      END;
END;

{***************************************************************************}
PROCEDURE HighIntensity;

VAR
  Regs : Registers;

BEGIN
    FillChar (regs, SizeOf(regs), 0); (* An initialization precaution *)
    regs.ah := $10;                   (* Function $10 *)
    regs.al := $03;                   (* Subfunction $03 *)
    regs.bl := $00;
    Intr ($10, regs);      (* ROM BIOS video driver interrupt *)
END;

{***************************************************************************}
PROCEDURE Beep (Pitch, Time : Integer);

BEGIN
  Sound (Pitch);
  Delay (Time);
  Nosound;
END;

{***************************************************************************}
PROCEDURE TcTb (Tc, Tb : byte);

BEGIN
  textcolor (Tc);
  textbackground (Tb);
END;

{***************************************************************************}
PROCEDURE WrtC (X, Y : byte;
   Chr: char);

BEGIN
  gotoXY (X, Y);
  write (Chr);
END;

{***************************************************************************}
PROCEDURE WrtS (X, Y : byte; Str : STRING);

BEGIN
  gotoXY (X, Y);
  write (Str);
END;

{***************************************************************************}
PROCEDURE WrtI (X, Y : byte; Int : integer);

BEGIN
  gotoXY (x, y);
  write (int);
END;

{***************************************************************************}
PROCEDURE WrtR (X, Y : byte; Reel : real; Field, Decimal : byte);

BEGIN
  gotoXY (X, Y);
  write (Reel:Field:Decimal);
END;

{***************************************************************************}
PROCEDURE FrmtI (X, Y : byte; Int: integer; Field : byte);

BEGIN
  gotoXY (X,Y);
  write (Int:Field);
END;

{***************************************************************************}
PROCEDURE Clear (Text, Backcolor, Ulx, Uly, Lrx, Lry : byte);

BEGIN
  TcTb (Text, Backcolor);
  window (Ulx,Uly,Lrx,Lry);
  clrscr;
END;

{***************************************************************************}
PROCEDURE CursorOn;

BEGIN
  {[F-]}
  INLINE($B4/3/$B7/0/$CD/16/$80/$E5/$DF/$B4/1/$CD/16);
  {[F+]}
END;

{***************************************************************************}
PROCEDURE CursorOff;

BEGIN
  {[F-]}
  INLINE($B4/3/$B7/0/$CD/16/$80/$CD/$20/$B4/1/$CD/16);
  {[F+]}
END;

{***************************************************************************}
PROCEDURE ShadowBack (X, Y, XLength, YLength : Byte);

VAR
  XShad,
  YShad : Word;
  Count : Byte;

BEGIN
  XLength := XLength SHL 1;
  XShad := ((X * 2) + (Y * 160) - 162) + ((YLength + 1) * 160) + 4;
  YShad := ((X * 2) + (Y * 160) - 162) + (XLength);
  IF odd(XShad) THEN
      inc(XShad);
  IF NOT odd(YShad) THEN
      inc(YShad);
  FOR Count := 1 TO XLength DO
    IF odd(Count) THEN
        mem[$B800:XShad + Count] := 8;
  FOR Count := 1 TO YLength DO
    BEGIN
      mem[$B800:YShad + (Count * 160)] := 8;
      mem[$B800:YShad + 2 + (Count * 160)] := 8
    END;
end;

{***************************************************************************}
PROCEDURE MakeWin (TC, BC, Ulx, Uly, Width, Height : byte; Border, Shadow : Boolean);

CONST
  Vertical    = #186;
  Horizontal  = #205;
  TopLeft     = #201;
  TopRight    = #187;
  BottomLeft  = #200;
  BottomRight = #188;

VAR
  Count : byte;

BEGIN
  window (1,1,80,25);;
  IF Shadow THEN
      ShadowBack (Ulx,Uly,Width,Height - 1);

  Clear (TC, BC, Ulx, Uly, Ulx + (Width - 1), Uly + (Height - 1));
  window (1,1,80,25);;
  IF Border THEN
      BEGIN
        FOR Count := (Ulx + 1) TO (Ulx + Width - 2) DO
          BEGIN
            WrtC (Count, Uly, Horizontal);
            WrtC (Count, (Uly + Height - 1), Horizontal);
          END;
        FOR Count := (Uly + 1) TO (Uly + Height - 2) DO
          BEGIN
           WrtC (Ulx, Count, Vertical);
           WrtC ((Ulx + Width - 1), Count, Vertical);
          END;
        WrtC (Ulx, Uly, Topleft);
        WrtC ((Ulx + Width - 1), Uly, TopRight);
        WrtC (Ulx, (Uly + Height - 1), BottomLeft);
        WrtC ((Ulx + Width - 1), (Uly + Height - 1), BottomRight);
      END;
  window (Ulx + 1, Uly + 1, (Ulx + Width - 2), (Uly  + Height - 2));
  TcTb (TC,BC);
END;

{***************************************************************************}
PROCEDURE CopyFile (FileName, Source, Destination : STRING);

VAR
  FromF,
  ToF          : FILE;
  NumRead,
  NumWritten   : Word;
  LargeBuf     : ^LargeBufType;

BEGIN
  new (LargeBuf);
  assign(FromF, Source + FileName);
  reset(FromF, 1);
  assign(ToF, Destination + FileName);
  rewrite(ToF, 1);
  REPEAT
    blockread(FromF, LargeBuf^, SizeOf(LargeBuf^), NumRead);
    blockwrite(ToF, LargeBuf^, NumRead, NumWritten);
  UNTIL (NumRead = 0) OR (NumWritten <> NumRead);
  close(FromF);
  close(ToF);
  dispose (LargeBuf);
END;

{***************************************************************************}
PROCEDURE MakeDir (NewDir : STRING);

VAR
  Try,
  Slash  : BYTE;
  Error  : WORD;
  TmpDir,
  IncDir,
  OurDir : STRING;

BEGIN
  GetDir (0,OurDir);
  WHILE NewDir[Length(NewDir)] = '\' DO DEC(NewDir[0]);
  IncDir := '';
  REPEAT
    Slash := Pos('\',NewDir);
    IF (Slash <> 0) THEN BEGIN
      IncDir := IncDir + Copy( NewDir, 1, Slash );
      NewDir := Copy( NewDir, Slash+1, Length(NewDir)-Slash ); END
    ELSE
      IncDir := IncDir + NewDir;
    TmpDir := IncDir;
    IF (Length(TmpDir) > 3) THEN
      WHILE TmpDir[Length(TmpDir)] = '\' DO DEC(TmpDir[0]);
    REPEAT
      {$I-} ChDir(TmpDir); {$I+}
      Error := IoResult;
      IF (Error <> 0) THEN BEGIN
        {$I-} MkDir(TmpDir); {$I+}
        Error := IoResult;
      END;
      IF (Error <> 0) THEN INC(Try) ELSE Try := 0;
    UNTIL (Error = 0) OR (Try > 3);
    IF (Error = 0) THEN WriteLn('"',TmpDir,'" -- okay');
  UNTIL (Slash = 0) OR (Error <> 0);
  ChDir (OurDir);
END;

{***************************************************************************}

END.
